/*!
 * controlwrap - v1.2.0
 * By hocti
 * Compiled Tue, 13 Aug 2024 07:49:44 UTC
 *
 * controlwrap is licensed under the MIT license.
 * http://www.opensource.org/licenses/mit-license
 */
import { EventEmitter } from 'eventemitter3';
import { dpad, dpadPress, xy, directionWrap, directSource } from 'gamepad_standardizer';
import * as gamepad_standardizer from 'gamepad_standardizer';
export { gamepad_standardizer };

declare enum buttonLayout {
    xbox = "xbox",
    sony = "sony",
    nintendo = "nintendo",
    unknown = "unknown",
    keyboard = "keyboard"
}

declare const UIButton: {
    confirm: string;
    escape: string;
    up: dpad.up;
    down: dpad.down;
    left: dpad.left;
    right: dpad.right;
};
declare enum UIButtonOptional {
    cancel = "cancel",
    info = "info",
    prevTab = "prevTab",
    nextTab = "nextTab"
}
declare enum ControlType {
    system = "system",
    KEYBOARD = "keyboard",
    GAMEPAD = "gamepad"
}
declare enum InputSource {
    system = "system",
    ui = "ui",
    player = "player",
    mouse = "mouse"
}
type ButtonState = {
    press: boolean;
    just: boolean;
    double: boolean;
    tap: boolean;
    pressFrame: number;
};
type gameInput = {
    dpad?: dpadPress & {
        numpad: number;
    } & xy;
    analog?: directionWrap;
    analog_right?: directionWrap;
    mixedDpad?: dpadPress & {
        numpad: number;
    } & xy;
    directionButton?: Record<dpad, ButtonState>;
    doublePressDirection?: boolean;
    button?: Record<string, ButtonState>;
};
type systemInput = {
    source?: ControlType;
    sourceIndex?: number;
    ui_tap: string[];
    ui_pressing: string[];
    ui_repeat: string[];
};
type Input = systemInput & gameInput;
type mappingRequirement = {
    direction: {
        dpad: boolean;
        analog: 0 | 1 | 2;
    };
    button: string[];
    optional?: string[];
    ui_button?: string[];
    ui_optional?: string[];
    ui_pair: Record<string, string>;
};
type mappingGroup = Record<string, string | string[]>;
type latestLayoutGroup = {
    name: string;
    layout: buttonLayout;
    mapping: mappingGroup;
};
type mouseInput = {
    xy?: xy;
    xyDelta?: xy;
    delta: number;
    buttons: Record<string, ButtonState>;
};
type InputGroup = {
    [InputSource.ui]: Input;
    [InputSource.system]: Input;
    [InputSource.player]: Record<number, Input>;
    [InputSource.mouse]?: mouseInput;
};
type ControllerInfo = {
    wrappedIndex: number;
    type: ControlType;
    index: number;
    name: string;
    profile?: string;
    isMainPlayer: boolean;
};
type ControllerProfile = {
    name: string;
    type: 'keyboard' | 'xinput' | 'gamepadName';
    gamepadName?: string;
    isDefault: boolean;
    mapping: mappingGroup;
};

interface IControllerMaster extends EventEmitter {
    init(): void;
    count(): number;
    getIndexs(): number[];
    flushAll(): Record<number, Input>;
    flushUI(): Record<number, systemInput>;
    getName(index: number): string;
    listenControllerIndexFromKeys(keys?: string[]): Promise<number>;
    listenKeyFromControllerIndex(index: number): Promise<number>;
    cancelListen(reason?: string): void;
    importRequirement(requirement: mappingRequirement): void;
    setAllMapping(index: number, mapping: mappingGroup): void;
    resetDefault(index: number): mappingGroup | undefined;
    getAllMapping(index: number): mappingGroup;
    setMapping(index: number, buttonName: string, keyCode: string): boolean;
    checkMapping(index: number, buttonName: string, keyCode: string): boolean;
    checkNotRepeat(index: number, buttonName: string, keyCode: string): boolean;
    getMappableKeys(index?: number): {
        buttons: string[];
        optional: string[];
    };
    renewSystemButtonLayout(index: number): latestLayoutGroup;
    vibration(index: number, duration: number, strongMagnitude: number, weakMagnitude: number): void;
}

type InputPreprocess = {
    direction?: Record<directSource, directionWrap | null>;
    button: Record<string, ButtonState>;
    directionButton: Record<string, ButtonState>;
};
declare class GamePadExtra {
    lastGamepad: Gamepad;
    private info;
    private btn_names;
    private directionAvailable;
    private buttons;
    private directionButton;
    analogThreshold: number;
    swapAB: boolean;
    layout: buttonLayout;
    getName(): string;
    getButtonNames(): (string | null)[];
    getDirectionAvailable(): Record<directSource, boolean>;
    constructor(gamepad: Gamepad);
    private unchangeCache;
    flush(gamepad: Gamepad, unchange?: boolean): InputPreprocess;
    vibration(duration?: number, strongMagnitude?: number, weakMagnitude?: number): Promise<GamepadHapticsResult> | undefined;
}

declare class ControlWrap extends EventEmitter implements IControllerMaster {
    protected static instance: ControlWrap;
    static getInstance(): ControlWrap;
    static isSupported(): Partial<Record<ControlType, boolean>>;
    static masterRecord: Partial<Record<ControlType, IControllerMaster>>;
    protected requirement: mappingRequirement | undefined;
    protected listening: Partial<Record<ControlType, boolean>>;
    init(listenTo?: Partial<Record<ControlType, boolean>>): void;
    protected listenToMouse: boolean;
    wrappedInit(requirement: mappingRequirement, defaultGamePadmapping: mappingGroup, defaultKeyboardmappings: mappingGroup[], keyboardCountum?: number, mouse?: boolean): void;
    importRequirement(requirement: mappingRequirement): void;
    setDefaultGamepadMapping(mapping: mappingGroup): void;
    setDefaultKeyboardMapping(index: number | undefined, mapping: mappingGroup): void;
    setPlayerCountFromKeyboard(num?: number): void;
    getPlayerCountFromKeyboard(): number;
    mainControllerIndex: number;
    allowMainPlayerUseAllController: boolean;
    private _lastInputIndex;
    private defaultButtonLayout;
    private lastButtonLayout;
    update(uiOnly?: boolean): InputGroup;
    flushUI(): Record<number, systemInput>;
    flushAll(): Record<number, Input>;
    private flushInner;
    renewSystemButtonLayout(index: number): latestLayoutGroup;
    getGamepadExtra(index: number): GamePadExtra | undefined;
    count(): number;
    getIndexs(): number[];
    parseIndex(index?: number): [ControlType, number];
    makeIndex(cType: ControlType, index?: number): number;
    listenControllerIndexFromKeys(keys?: string[]): Promise<number>;
    listenKeyFromControllerIndex(index: number): Promise<number>;
    cancelListen(reason?: string): void;
    getAllMapping(index: number): mappingGroup;
    checkMapping(index: number, buttonName: string, keyCode: string): boolean;
    setMapping(index: number, buttonName: string, keyCode: string): boolean;
    setAllMapping(index: number, mapping: mappingGroup): void;
    resetDefault(index: number): mappingGroup | undefined;
    getMappableKeys(index?: number): {
        buttons: string[];
        optional: string[];
    };
    checkNotRepeat(index: number, buttonName: string, keyCode: string): boolean;
    getName(index: number): string;
    vibration(index: number, duration?: number, strongMagnitude?: number, weakMagnitude?: number): void;
}

declare enum listenStatus {
    none = 0,
    anyKey = 1,
    ControllerIndexFromKeys = 2,
    raw = 3
}
declare abstract class InputDeviceMaster extends EventEmitter {
    protected requirement: mappingRequirement | undefined;
    protected currentMapping: Record<number, mappingGroup>;
    protected listenMode: listenStatus;
    protected listenKeys: string[];
    listenControllerIndexFromKeys(keys?: string[]): Promise<number>;
    cancelListen(reason?: string): void;
    getAllMapping(id: number): mappingGroup;
    abstract checkMapping(id: number, buttonName: string, keyCode: string): boolean;
    setAllMapping(id: number, mapping: mappingGroup): void;
    setMapping(id: number, buttonName: string, keyCode: string): boolean;
    getMappableKeys(id?: number): {
        buttons: string[];
        optional: string[];
    };
    vibration(id: number, duration: number, strongMagnitude: number, weakMagnitude: number): void;
}

declare enum Operation {
    Plus = "+",
    Minus = "-"
}
type RawEvent = {
    index: number;
    type: 'axes';
    axes_index: number;
    Operation: Operation;
} | {
    index: number;
    type: 'hat';
    axes_index: number;
    HNum: number;
} | {
    index: number;
    type: 'button';
    button_index: number;
};
declare class GamepadMaster extends InputDeviceMaster implements IControllerMaster {
    protected static instance: GamepadMaster;
    static getInstance(): GamepadMaster;
    static isSupported(): boolean;
    private inited;
    private gamePads;
    count(): number;
    getIndexs(): number[];
    init(): void;
    getGamepadExtra(index: number): GamePadExtra | undefined;
    getName(index: number): string;
    private addGP;
    private removeGP;
    flushAll(): Record<number, Input>;
    flushUI(): Record<number, systemInput>;
    private flushInner;
    private getPairFromBtnName;
    private getPairFromUiBtnName;
    renewSystemButtonLayout(index: number): latestLayoutGroup;
    private processInput;
    private listenRawGamepad;
    private listenRawType;
    saveAllRelaseGPState(index: number): void;
    listenRaw(index?: number, button?: boolean, axes?: boolean): Promise<RawEvent>;
    private processRaw;
    private listenDown;
    private listeningIndex;
    listenKeyFromControllerIndex(index: number): Promise<number>;
    private systemEscape;
    importRequirement(requirement: mappingRequirement): void;
    protected defaultStandardMapping: mappingGroup;
    resetDefault(index: number): mappingGroup | undefined;
    setDefaultMapping(mapping: mappingGroup): void;
    checkMapping(index: number, buttonName: string, keyCode: string): boolean;
    checkNotRepeat(index: number, buttonName: string, keyCode: string): boolean;
    vibration(index: number, duration?: number, strongMagnitude?: number, weakMagnitude?: number): void;
}

declare class KeyboardMaster extends InputDeviceMaster implements IControllerMaster {
    protected static instance: KeyboardMaster;
    static getInstance(): KeyboardMaster;
    static isSupported(): boolean;
    private downKey;
    private buttons;
    private inited;
    init(): void;
    importRequirement(requirement: mappingRequirement): void;
    private totalKeyboard;
    count(): number;
    getIndexs(): number[];
    getName(index: number): string;
    protected getInput(id: number, keyStatus: Record<string, ButtonState>): Input;
    protected getSystemInput(keyStatus: Record<string, ButtonState>): Input;
    flushAll(): Record<number, Input>;
    private unchangeCache;
    flushUI(): Record<number, systemInput>;
    listenControllerIndexFromKeys(keys?: string[]): Promise<number>;
    private listenDown;
    listenKeyFromControllerIndex(): Promise<number>;
    private defaultMapping;
    resetDefault(index: number): mappingGroup | undefined;
    setDefaultMapping(mapping: mappingGroup): void;
    setDefaultKeyboardMapping(id: number, mapping: mappingGroup): void;
    checkMapping(id: number, buttonName: string, keyCode: string): boolean;
    checkNotRepeat(id: number, buttonName: string, keyCode: string): boolean;
    getMappableKeys(id?: number): {
        buttons: string[];
        optional: string[];
    };
    renewSystemButtonLayout(index: number): latestLayoutGroup;
    getPairMapping(id: number): mappingGroup;
    setPlayerCountFromKeyboard(num: number): void;
}

declare class PlayerIndexMapper extends EventEmitter {
    minPlayer: number;
    maxPlayer: number;
    block: {
        sourceIndex: number;
        ready: boolean;
    }[];
    readyToStart: boolean;
    constructor(maxPlayer?: number, minPlayer?: number);
    setMinMax(maxPlayer?: number, minPlayer?: number): void;
    setBlock(sourceIndex: number, blockId?: number): number;
    removeBlock(blockId: number): void;
    setBlockReady(blockId: number, ready?: boolean): void;
    dispose(): void;
}

declare function tapOrRepeat(ip: Input, key: string, only?: boolean): boolean;
declare function tap(ip: Input, key: string, only?: boolean): boolean;
declare function pressing(ip: Input, key: string): boolean;
declare function YES(ip: Input): boolean;
declare function NO(ip: Input): boolean;
declare function PAUSE(ip: Input): boolean;
declare function UD(ip: Input): number;
declare function LR(ip: Input): number;
declare function L1R1(ip: Input): number;
declare function UDLRpressing(ip: Input): xy;
declare function UDLR(ip: Input): xy;
declare function mouseTap(ips: InputGroup, keyName: string): boolean;
declare function click(ips: InputGroup): boolean;

type fightingInputCommand = {
    directions: string;
    buttons: string[] | string;
    needAllbuttons?: boolean;
    directionTimes?: number;
};
declare function fightingInput(command: fightingInputCommand, inputs: string[], flip?: boolean): boolean;
declare function fightingInputMulti(commands: Record<string, fightingInputCommand>, inputs: string[], flip?: boolean): Record<string, boolean>;
declare function str2Command(str: string): fightingInputCommand | undefined;

declare class pressHistory {
    protected static instance: pressHistory;
    static getInstance(): pressHistory;
    historys: Record<number, Record<number, gameInput>>;
    historys_lite: Record<number, Record<number, string[]>>;
    autoFrameNumber: number;
    maxHistory: number;
    cleanAll(): void;
    cleanPlayer(playerID: number): void;
    addHistory(inputs: Record<number, Input>): void;
    getHistoryFromPlayer(playerID: number, maxFrame?: number): gameInput[];
    getHistoryLiteFromPlayer(playerID: number, maxFrame?: number): string[][];
    getShorten(playerID: number, maxFrame?: number): string;
    doublePressDirection(playerID: number, maxFrame?: number): number;
    doublePress(playerID: number, buttonName: string, maxFrame?: number): boolean;
}

declare function getUITap(bs: ButtonState | undefined): boolean;
declare function getUIsTap(bss: Record<string, ButtonState>): Record<string, boolean>;
declare function joinState(a: ButtonState, b: ButtonState): ButtonState;
declare function setMinDoubleFrame(frame: number): void;

declare class MouseMaster extends EventEmitter {
    protected static instance: MouseMaster;
    static getInstance(): MouseMaster;
    static isSupported(): boolean;
    protected lastxy: xy;
    protected latestxy: xy;
    protected delta: number;
    protected lastPress: boolean[];
    private buttons;
    private inited;
    init(): void;
    protected wEvent(event: WheelEvent): void;
    protected mEvent(event: MouseEvent): void;
    protected onRightClick(e: MouseEvent): void;
    protected onBlur(e: FocusEvent): void;
    listenXY: boolean;
    flush(): mouseInput;
    blockRightClick: boolean;
    blockPrevNext: boolean;
    blockMiddle: boolean;
    hideCursor(): void;
    showCursor(): void;
    getCursorDisplay(): boolean;
}

export { type ButtonState, ControlType, ControlWrap, type ControllerInfo, type ControllerProfile, GamepadMaster, type Input, type InputGroup, InputSource, KeyboardMaster, L1R1, LR, MouseMaster, NO, PAUSE, PlayerIndexMapper, UD, UDLR, UDLRpressing, UIButton, UIButtonOptional, YES, buttonLayout, click, fightingInput, fightingInputMulti, getUITap, getUIsTap, joinState, type mappingRequirement, mouseTap, pressHistory, pressing, setMinDoubleFrame, str2Command, tap, tapOrRepeat };
